"use strict";
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const execa_1 = __importDefault(require("execa"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const mkdirp = __importStar(require("mkdirp"));
const rimraf = __importStar(require("rimraf"));
const util_1 = require("./util");
const logger = util_1.getLogger();
const configPath = process.argv[2] || path.join(process.cwd(), "projects.yml");
if (!fs.existsSync(configPath)) {
    logger.error(`config path doesnt exist:\n ${configPath}`);
    process.exit(1);
}
const { baseDir, projects, typedefs = null } = util_1.getConfig(configPath);
const typeDefs = typedefs;
async function setup() {
    const cwd = path.join(process.cwd(), baseDir);
    let isGitRepo = true;
    try {
        await execa_1.default("git", ["status"], { cwd });
    }
    catch (err) {
        isGitRepo = false;
    }
    if (isGitRepo) {
        logger.error(`cannot intialize in a git repository`);
        process.exit(1);
        return;
    }
    if (!fs.existsSync(cwd)) {
        logger.warn(`creating working directory`);
        await mkdirp.sync(baseDir);
    }
}
async function cloneProject(remote, projectDir) {
    if (!fs.existsSync(projectDir)) {
        return execa_1.default("git", ["clone", remote, projectDir]);
    }
    logger.warn(`${projectDir} already exists`);
}
async function cloneTypeDefinitions() {
    var e_1, _a;
    try {
        for (var _b = __asyncValues(Object.entries(typeDefs)), _c; _c = await _b.next(), !_c.done;) {
            let [typeDefName, typeInfo] = _c.value;
            const typeDefPath = path.join(process.cwd(), baseDir, "@types", typeDefName);
            if (!fs.existsSync(typeDefPath)) {
                await mkdirp.sync(typeDefPath);
            }
            if (fs.existsSync(path.join(typeDefPath, ".git"))) {
                return;
            }
            await execa_1.default("git", ["init"], { cwd: typeDefPath });
            await mkdirp.sync(path.join(typeDefPath, ".git/info"));
            await execa_1.default("git", ["remote", "add", "-f", "origin", typeInfo.remote], { cwd: typeDefPath });
            await execa_1.default("git", ["config", "core.sparseCheckout", "true"], { cwd: typeDefPath });
            await fs.writeFileSync(path.join(typeDefPath, ".git/info/sparse-checkout"), `types/${typeInfo.typeName}`, "utf8");
            await execa_1.default("git", ["checkout", typeInfo.branch], { cwd: typeDefPath });
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) await _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
}
async function checkoutBranch(projectDir, branch) {
    return execa_1.default("git", ["checkout", branch], { cwd: projectDir });
}
async function each(fn) {
    var e_2, _a;
    try {
        for (var projects_1 = __asyncValues(projects), projects_1_1; projects_1_1 = await projects_1.next(), !projects_1_1.done;) {
            const project = projects_1_1.value;
            const projectDir = path.join(process.cwd(), baseDir, project.package);
            await fn(projectDir, project);
        }
    }
    catch (e_2_1) { e_2 = { error: e_2_1 }; }
    finally {
        try {
            if (projects_1_1 && !projects_1_1.done && (_a = projects_1.return)) await _a.call(projects_1);
        }
        finally { if (e_2) throw e_2.error; }
    }
}
async function setupGitRepos(projectDir, project) {
    await cloneProject(project.remote, projectDir);
    await checkoutBranch(projectDir, project.branch);
    logger.info(`cloned and checked out ${project.package}#${project.branch}`);
}
async function installDependencies(projectDir, project) {
    await execa_1.default("yarn", ["install", "--ignore-scripts"], { cwd: projectDir });
    logger.info(`installed dependencies for ${project.package}`);
}
async function buildProject(projectDir, project) {
    if (project.lerna) {
        logger.info(`built ${project.package}`);
        return execa_1.default("lerna", ["run", "build"], { cwd: projectDir });
    }
    logger.info(`built ${project.package}`);
    return execa_1.default("yarn", ["build"], { cwd: projectDir });
}
async function linkSelf(projectDir, project, relink = false) {
    let cwd = projectDir;
    if (project.linkFrom) {
        cwd = path.join(projectDir, project.linkFrom);
    }
    if (relink) {
        await execa_1.default("yarn", ["unlink"], { cwd });
    }
    if (project.lerna) {
        return execa_1.default("lerna", ["exec", "yarn", "link"], { cwd });
    }
    return execa_1.default("yarn", ["link"], { cwd });
}
async function linkDependencies(projectDir, project) {
    if (project.links) {
        if (project.lerna) {
            await execa_1.default("lerna", ["exec", "--", "yarn", "link", ...project.links], { cwd: projectDir });
        }
        else {
            await execa_1.default("yarn", ["link", ...project.links], { cwd: projectDir });
        }
        logger.info(`linked dependencies for ${project.package}`);
        return;
    }
}
async function linkTypes(projectDir, project) {
    if (project.types) {
        for (let type of project.types) {
            const typeSymbolicPath = path.join(projectDir, `node_modules/@types/${type}`);
            const typePath = path.join(process.cwd(), baseDir, "@types", type, "types", typeDefs[type].typeName);
            rimraf.sync(typeSymbolicPath);
            return fs.symlinkSync(typePath, typeSymbolicPath, "dir");
        }
    }
}
async function setupProject(projectDir, project) {
    await installDependencies(projectDir, project);
    if (project.types) {
        await linkTypes(projectDir, project);
    }
    await linkDependencies(projectDir, project);
    await buildProject(projectDir, project);
    await linkSelf(projectDir, project);
}
(async () => {
    try {
        await setup();
        await each(setupGitRepos);
        logger.warn("setting up typeDefs, this could take a while... and a lot of bandwith...");
        await cloneTypeDefinitions();
        logger.info("type defs setup");
        await each(setupProject);
    }
    catch (err) {
        logger.error(err);
    }
})();
